#!/usr/bin/python
# -*- coding: utf-8 -*-
# 
#   FILE: Oscar2016TweetsDB.py
#   DATE: January, 2016
#   Author: David W. McDonald
#
#   An object for managing the database of the 2016 Oscars 
#
#   Copyright by Author. All rights reserved. Not for reuse without
#   express permissions.
#
#
from hcde.data.db.base.TweetsDB import TweetsDB
from hcde.data.db.oscar2016.Oscar2016TweetObj import Oscar2016TweetObj
from hcde.data.db.oscar2016.Oscar2016FriendObj import Oscar2016FriendObj
from hcde.data.db.oscar2016.Oscar2016FollowerObj import Oscar2016FollowerObj
from hcde.data.db.oscar2016.Oscar2016UserObj import Oscar2016UserObj
from hcde.data.db.oscar2016.Oscar2016UserMetaObj import Oscar2016UserMetaObj
from sqlalchemy.orm import mapper, class_mapper, clear_mappers
from sqlalchemy.orm.exc import UnmappedClassError
import sys

class Oscar2016TweetsDB(TweetsDB):
    def __init__(self, config=None):
        TweetsDB.__init__(self, config=config)

        # Seems to fix a intermittent 'mapper exception'
        # NOTE: this probably prevents having two DBs 'open' at the same time
        clear_mappers()
        try:
            self.tweet_mapper = class_mapper(Oscar2016TweetObj)
        except UnmappedClassError:
            self.tweet_mapper = mapper(Oscar2016TweetObj,self.tweet_table)
        
        try:
            self.user_mapper = class_mapper(Oscar2016UserObj)
        except UnmappedClassError:
            self.user_mapper = mapper(Oscar2016UserObj,self.user_table)

        try:
            self.user_meta_mapper = class_mapper(Oscar2016UserMetaObj)
        except UnmappedClassError:
            self.user_meta_mapper = mapper(Oscar2016UserMetaObj,self.user_meta_table)

        try:
            self.friend_mapper = class_mapper(Oscar2016UserMetaObj)
        except UnmappedClassError:
            self.friend_mapper = mapper(Oscar2016UserMetaObj,self.friends_table)

        try:
            self.follower_mapper = class_mapper(Oscar2016FollowerObj)
        except UnmappedClassError:
            self.follower_mapper = mapper(Oscar2016FollowerObj,self.followers_table)

##
# New object creation routines
##
    ## Create "tweet" objects and records
    def new_tweet_table_item(self, rec=None):
        nto = Oscar2016TweetObj()
        if( rec ):
            return nto.from_dict(rec)
        return nto


    ## Create "user" objects and records
    def new_user_table_item(self, rec=None):
        uto = Oscar2016UserObj()
        if( rec ):
            return uto.from_dict(rec)
        return uto

    ## Create "user_meta" objects and records
    def new_user_meta_table_item(self, rec=None):
        umto = Oscar2016UserMetaObj()
        if( rec ):
            return umto.from_dict(rec)
        return umto

    ## Create "friend" objects and records
    def new_friend_table_item(self, rec=None):
        nf = Oscar2016FriendObj()
        if( rec ):
            return nf.from_dict(rec)
        return nf

    ## Create "follower" objects and records
    def new_follower_table_item(self, rec=None):
        nf = Oscar2016FollowerObj()
        if( rec ):
            return nf.from_dict(rec)
        return nf

##
# straight forward query types
##
    ##
    ## Query the tweet table
    ##
    def query_tweet_table_by_tweet_id(self, tid):
        t = self.session.query(Oscar2016TweetObj).filter(Oscar2016TweetObj.tweet_id==tid).all()
        return t

    def query_tweet_table_by_username(self, uname=None, start_date=None, end_date=None,
                                            in_order=False):
        q = self._tweet_table_date_range(start_date=start_date, end_date=end_date)
        q = q.filter(Oscar2016TweetObj.from_user==uname)
        if( in_order ):
            q = q.order_by(Oscar2016TweetObj.created_at)
        tlist = q.all()
        return tlist

    def query_tweet_table_by_user_id(self, uid=None, start_date=None, end_date=None,
                                           in_order=False):
        q = self._tweet_table_date_range(start_date=start_date, end_date=end_date)
        q = q.filter(Oscar2016TweetObj.from_user_id==uid)
        if( in_order ):
            q = q.order_by(Oscar2016TweetObj.created_at)
        tlist = q.all()
        return tlist

    def query_tweet_table_by_tweet_substr(self, substr=None, start_date=None, end_date=None,
                                                in_order=False):
        qtext = "%"+substr+"%"
        q = self._tweet_table_date_range(start_date=start_date, end_date=end_date)
        q = q.filter(Oscar2016TweetObj.tweet_text.like(qtext))
        if( in_order ):
            q = q.order_by(Oscar2016TweetObj.created_at)
        tlist = q.all()
        return tlist

    def query_tweet_table_by_date_range(self, start_date=None, end_date=None, in_order=True):
        q = self._tweet_table_date_range(start_date=start_date, end_date=end_date)
        if( in_order ):
            q = q.order_by(Oscar2016TweetObj.created_at)
        tlist = q.all()
        return tlist

    def _tweet_table_date_range(self, start_date=None, end_date=None):
        q = self.session.query(Oscar2016TweetObj)
        if( start_date and end_date ):
            q = q.filter(Oscar2016TweetObj.created_at>=start_date)
            q = q.filter(Oscar2016TweetObj.created_at<end_date)
        elif( start_date ):
            q = q.filter(Oscar2016TweetObj.created_at>=start_date)
        elif( end_date ):
            q = q.filter(Oscar2016TweetObj.created_at<end_date)
        else:
            pass
        return q


    ##
    ## Query the user table
    ##
    def query_user_table_by_record_id(self, rid=None, rid2=None):
        if( rid2 ):
            q = self.session.query(Oscar2016UserObj).filter(Oscar2016UserObj.rid>=rid)
            tlist = q.filter(Oscar2016UserObj.rid<rid2).all()
        else:
            tlist = self.session.query(Oscar2016UserObj).filter(Oscar2016UserObj.rid==rid).all()
        return tlist

    def query_user_table_by_fullname(self, sname=None, in_order=True):
        q = self.session.query(Oscar2016UserObj).filter(Oscar2016UserObj.screen_name==sname)
        if( in_order ):
            q = q.order_by(Oscar2016UserObj.collect_id)
        tlist = q.all()
        return tlist

    def query_user_table_by_screenname(self, sname=None, in_order=True):
        q = self.session.query(Oscar2016UserObj).filter(Oscar2016UserObj.screen_name==sname)
        if( in_order ):
            q = q.order_by(Oscar2016UserObj.collect_id)
        tlist = q.all()
        return tlist

    def query_user_table_by_username(self, uname=None, in_order=True):
        q = self.session.query(Oscar2016UserObj).filter(Oscar2016UserObj.user_name==uname)
        if( in_order ):
            q = q.order_by(Oscar2016UserObj.collect_id)
        tlist = q.all()
        return tlist

    def query_user_table_by_user_id(self, uid=None, in_order=True):
        tlist = self.session.query(Oscar2016UserObj).filter(Oscar2016UserObj.user_id==uid)
        if( in_order ):
            q = q.order_by(Oscar2016UserObj.collect_id)
        tlist = q.all()
        return tlist

    ##
    ## Query the user_meta table
    ##
    def query_user_meta_table_by_record_id(self, rid=None, rid2=None):
        if( rid2 ):
            q = self.session.query(Oscar2016UserMetaObj).filter(Oscar2016UserMetaObj.rid>=rid)
            tlist = q.filter(Oscar2016UserMetaObj.rid<rid2).all()
        else:
            tlist = self.session.query(Oscar2016UserMetaObj).filter(Oscar2016UserMetaObj.rid==rid).all()
        return tlist

    def query_user_meta_table_by_fullname(self, sname=None, in_order=True):
        q = self.session.query(Oscar2016UserMetaObj).filter(Oscar2016UserMetaObj.screen_name==sname)
        if( in_order ):
            q = q.order_by(Oscar2016UserMetaObj.collect_id)
        tlist = q.all()
        return tlist

    def query_user_meta_table_by_screenname(self, sname=None, in_order=True):
        q = self.session.query(Oscar2016UserMetaObj).filter(Oscar2016UserMetaObj.screen_name==sname)
        if( in_order ):
            q = q.order_by(Oscar2016UserMetaObj.collect_id)
        tlist = q.all()
        return tlist

    def query_user_meta_table_by_username(self, uname=None, in_order=True):
        q = self.session.query(Oscar2016UserMetaObj).filter(Oscar2016UserMetaObj.user_name==uname)
        if( in_order ):
            q = q.order_by(Oscar2016UserMetaObj.collect_id)
        tlist = q.all()
        return tlist

    def query_user_meta_table_by_user_id(self, uid=None, in_order=True):
        tlist = self.session.query(Oscar2016UserMetaObj).filter(Oscar2016UserMetaObj.user_id==uid)
        if( in_order ):
            q = q.order_by(Oscar2016UserMetaObj.collect_id)
        tlist = q.all()
        return tlist


    ##
    ## Query the friends table
    ##
    def query_friends_by_username(self, uname=None, fname=None, cid=-1):
        q = self.session.query(Oscar2016FriendObj).filter(Oscar2016FriendObj.user==uname)
        if( fname ):
            q = q.filter(Oscar2016FriendObj.friend==fname)
        if( cid>0 ):
            q = q.filter(Oscar2016FriendObj.collect_id==cid)
        flist = q.all()
        return flist

    def query_friends_by_user_id(self, uid=None, fid=None, cid=-1):
        q = self.session.query(Oscar2016FriendObj).filter(Oscar2016FriendObj.user_id==uid)
        if( fid ):
            q = q.filter(Oscar2016FriendObj.friend_id==fid)
        if( cid>0 ):
            q = q.filter(Oscar2016FriendObj.collect_id==cid)
        flist = q.all()
        return flist

    ##
    ## Query the followers table
    ##
    def query_followers_by_username(self, uname=None, fname=None, cid=-1):
        q = self.session.query(Oscar2016FollowerObj).filter(Oscar2016FollowerObj.user==uname)
        if( fname ):
            q = q.filter(Oscar2016FollowerObj.follower==fname)
        if( cid>0 ):
            q = q.filter(Oscar2016FollowerObj.collect_id==cid)
        flist = q.all()
        return flist

    def query_followers_by_user_id(self, uid=None, fid=None, cid=-1):
        q = self.session.query(Oscar2016FollowerObj).filter(Oscar2016FollowerObj.user_id==uid)
        if( fid ):
            q = q.filter(Oscar2016FollowerObj.follower_id==fid)
        if( cid>0 ):
            q = q.filter(Oscar2016FollowerObj.collect_id==cid)
        flist = q.all()
        return flist



