#!/usr/bin/python
# -*- coding: utf-8 -*-
# 
#   FILE: build_graph.py
#   DATE: December, 2013
#   Author: David W. McDonald
#
#   Example of processing tweets to build a graph structure and save
#   that graph to visualize in another tool.
#
#   Copyright by Author. All rights reserved. Not for reuse without
#   express permissions.
#
import sys, gc, time, string, json, pickle, random
from datetime import datetime, timedelta
from hcde.data.db.base.dbConfig import DBConfiguration
from hcde.data.db.elect2012.settings_db import *
from hcde.data.db.elect2012.Elect2012TweetsDB import Elect2012TweetsDB as DB
from hcde.data.db.elect2012.Elect2012TweetObj import Elect2012TweetObj
from hcde.utils.tweet_entities import tweet_entities
from hcde.data.election_2012.constants import *
import networkx as nx



def query_date(db=None, date=None, dur=1, by_hour=False):
    result_list = []
    if( by_hour ):
        delta = timedelta(hours=1)
    else:
        delta = timedelta(days=1)
    dt2 = date + (dur*delta)
    start_date = date.strftime("%Y%m%d%H%M%S")
    end_date = dt2.strftime("%Y%m%d%H%M%S")
    # these two force it to start and end at a *day* boundary by
    # ignoring any HoursMinutesSeconds in the daytime string
    #start_date = date.strftime("%Y%m%d000000")
    #end_date = dt2.strftime("%Y%m%d000000")
    try:
        result_list = db.query_tweet_table_by_date_range(start_date=start_date,
                                                         end_date=end_date,
                                                         in_order=True)
    except Exception, e:
        print "EXCEPTION when running query!"
        print e
        result_list = []
    return {'tweet_list':result_list, 
            'query_date':date, 
            'start_date_str':start_date, 
            'end_date_str':end_date,
            'duration':dur}



##
#   Add a node to the graph, if this is a weighted graph then we keep
#   indexing that weight, otherwise, just add a given edge once
#
def add_node(graph=None, u1=None, u2=None, tweet=None, weighted=False):
    if( u1 and u2 ):        
        edge_dict = graph.get_edge_data(u1, u2)
        # if the edge attribute dict is empty, then just add the edge
        # with the given conditions (parameters)
        if( not edge_dict ):
            #graph.add_edge(u1, u2, weight=1)
            graph.add_edge(u1, u2, {"weight":1, "tweet_ids": tweet.tweet_id_str})
        else:
            if( weighted ):
                graph[u1][u2]['weight']+=1
                tids = graph[u1][u2]['tweet_ids']
                graph[u1][u2]['tweet_ids'] = tids+","+tweet.tweet_id_str



##
#   This handles the odd implementation that node attributes look
#   like other "nodes" in the graph to networkx. Given a node this will
#   return the proper list of node neighbors
#
def get_neighbors(graph=None, node=None):
    result = []
    if( node ):
        neighbors = graph.neighbors(node)
        for n in neighbors:
            try:
                neighbor = graph[n]
                result.append(n)
            except:
                pass
    return result



##
#   Code to trim lone edges/nodes from the graph.
#
def single_edge_cut(graph=None, times=1, ecuts=0, ncuts=0):
    #print "single_edge_cut: starting pass %d"%(times)
    made_cut = False
    result = [times, ecuts, ncuts]
    gn = graph.nodes()
    for node in gn:
        neighbors = get_neighbors(graph=graph,node=node)
        if( len(neighbors)==0 ):
            # this node is an isolated node, remove it
            made_cut = True
            ncuts += 1
            graph.remove_node(node)
        elif( len(neighbors)==1 ):
            # this node has exactly one edge, remove the edge
            # then remove the node
            made_cut = True
            ecuts += 1
            ncuts += 1
            graph.remove_edge(node,neighbors[0])
            graph.remove_node(node)
        else:
            pass

    # If made_cut is false then we didn't cut any edges and we should simply
    # return the graph and the number of times we have been through the graph
    # However, if made_cut is true, then we cut some edges and we need to 
    # recursively check the graph to see if there are any other edges and nodes
    # which need to be removed from the graph
    if( made_cut ):
        times += 1
        result = single_edge_cut(graph=graph, times=times, ecuts=ecuts, ncuts=ncuts)
    
    return result



##
#   Code to create a new graph based on the largest component
#
def find_largest_component(graph=None, report=False):
    if( graph.is_directed() ): return graph
    ng = graph
    comps = nx.connected_components(graph)
    largest_comp = []
    for c in comps:
        if( len(c) > len(largest_comp) ):
            largest_comp = c
            if( report ):
                #print "Largest Component (%d)"%(len(largest_comp)),largest_comp
                print "Largest Component (%d)"%(len(largest_comp))
    ng = nx.Graph(graph.subgraph(largest_comp))
    return ng



##
#   Graph the tweets as a function of who we find as a retweet user
#   Basically, just create a NetworkX graph with nodes as user names and
#   edges as a function of the retweet
#
def graph_tweets(db=None, start_date=None, dur=1, digraph=False, weighted=False, report=False):
    rt_count = 0
    
    # start by creating a graph 
    if( digraph ):
        g = nx.DiGraph()
    else:
        g = nx.Graph()
    
    # query the database to get a set (list) of tweets
    result = query_date(db=db, date=start_date, dur=dur)
    tweet_list = result['tweet_list']
    
    if( report ):
        print "Found %d tweets."%(len(tweet_list))
        print "Graph Edges:"
    
    # now iterate through the list of the tweet objects
    for tweet in tweet_list:
        tweet_stuff = tweet_entities(tweet_text=tweet.tweet_text)
        # first test to see if this is even a retweet!
        if( tweet_stuff['is_retweet'] ):
            rt_count += 1
            # we need to create nodes of user names and edges between those names
            user = tweet.from_user
            rt_user_name = tweet_stuff['guessed_retweet_from_user_name']
            if( rt_user_name ):
                if( report ):
                    print "\t",user.encode('utf-8'),
                    print "-->",rt_user_name.encode('utf-8')
                add_node(graph=g, u1=user, u2=rt_user_name,
                         tweet=tweet, weighted=weighted)
        
    if( report ):
        print "Graph has %d nodes and %d edges."%(len(g.nodes()),len(g.edges()))
        
    return g



##
#   This is a very simple "dot" file writer. Often the networkx verions does not
#   work correctly, which results in the manual version (code below the except
#   clause). This will fail on certain Unicode node names!
#
def write_dot_file(graph=None,fname=None):
    if( not graph ):
        print "Must provide a graph object to save!"
        return
    if( not fname ):
        fname = "simple_graph_file.dot"
    try:
        nx.drawing.write_dot(graph,fname)
    except ImportError, e:
        dot = ['"%s" -> "%s"'%(n1, n2) for n1,n2 in graph.edges()]
        dotenc = [ item.encode('utf-8') for item in dot ]
        f = open(fname,"w")
        f.write('strict digraph {\n%s\n}'%(';\n'.join(dotenc),))
        f.close()



def parse_date(dstr=None):
    date = None
    try:
        date = datetime.strptime(dstr,"%Y%m%d")
    except:
        try:
            date = datetime.strptime(dstr,"%d-%m-%Y")
        except:
            try:
                date = datetime.strptime(dstr,"%d/%m/%Y")
            except:
                print "Can't parse that date."
                date = None
    return date



def parse_params(argv):
    dt = None          # date for the doc
    dt_str = None      # date as a string entered by the user
    dur = 1            # duration
    digraph = False    # make this a digraph
    weighted = False   # weight the edges of the graph
    cut = False        # perform a recursive single edge cut of the graph
    comp = False       # operate on the largest connected component
    graphml = True     # if saving a file save it in graphml or dot format
    report = True      # report progress
    fname = None       # file name for the resulting graph
    pc = 1
    while( pc < len(argv) ):
        param = argv[pc]
        if( param == "-date"):
            pc += 1
            dt_str = argv[pc]
            dt = parse_date(dt_str)
        if( param == "-dur"):
            pc += 1
            dur = int(argv[pc])
        if( param == "-save"):
            pc += 1
            fname = argv[pc]
        if( param == "-weighted"):
            weighted = True
        if( param == "-digraph"):
            digraph = True
        if( param == "-edge_cut"):
            cut = True
        if( param == "-comp"):
            comp = True
        if( param == "-graphml"):
            graphml = True
        if( param == "-dot"):
            graphml = False
        if( param == "-report"):
            report = True
        if( param == "-no_report"):
            report = False
        pc += 1
    return {'date':dt, 'dt_str':dt_str, 'fname':fname, 'duration':dur,
            'weighted':weighted, 'digraph':digraph, 'cut':cut, 'comp':comp,
            'graphml':graphml, 'report':report }



def usage(prog):
    print "USAGE: %s -date <date> -save <filename> [-dur <days>] [-digraph] [-weighted] [-edge_cut] [-comp] [-dot | -graphml] [-report | -no_report]"%(prog)
    sys.exit(0)


# Some simple examples of using this at the command line
#
#   python build_graph.py -date 20120901
#   python build_graph.py -date 20120901 -dur 2
#   python build_graph.py -date 20120901 -no_report
#   python build_graph.py -date 20121001 -save election2012 -dot
#   python build_graph.py -date 20121001 -save election2012 -graphml -edge_cut
#   python build_graph.py -date 20121001 -dur 3 -save election2012 -graphml -edge_cut
#   python build_graph.py -date 20121001 -dur 7 -save election2012 -graphml -edge_cut

def main(argv):
    if len(argv) < 3:
        usage(sys.argv[0])
    params = parse_params(argv)

    if( params['report'] ):
        print "Got parameters"
        print params

    if( params['report'] ):
        print "Preparing Database Configuration"
    config = DBConfiguration(db_settings=DATABASE_SETTINGS['default'])
    #config = DBConfiguration(db_settings=DATABASE_SETTINGS['main_db'])

    if( params['report'] ):
        print config
        print "Opening Database"
    
    # Open the database with the specific configuration
    db = DB(config=config)
    
    g = graph_tweets(db=db,start_date=params['date'],
                     dur=params['duration'],
                     digraph=params['digraph'],
                     weighted=params['weighted'],
                     report=params['report'])

    if( params['comp'] ):
        if( params['report'] ):
            print "Finding largest connected component"
        g = find_largest_component(graph=g,report=params['report'])
        if( params['report'] ):
            print "New Graph has %d nodes and %d edges."%(len(g.nodes()),len(g.edges()))
    
    if( params['cut'] ):
        if( params['report'] ):
            print "Performing recursive single edge cut."
        result = single_edge_cut(graph=g)
        if( params['report'] ):
            print "Made %d passes through the graph, cut %d edges and %d nodes."%(result[0],result[1],result[2])
            print "Graph has %d nodes and %d edges."%(len(g.nodes()),len(g.edges()))
    

    # Save the resulting graph so it can be rendered with another tool
    if( params['fname'] ):
        dt_str = params['date'].strftime("%Y%m%d")
        fname = params['fname']+"-"+dt_str+"-dur%02d"%(params['duration'])
        if( params['digraph'] ):
            fname = fname+"-digraph"
        if( params['weighted'] ):
            fname = fname+"-weighted"
        if( params['comp'] ):
            fname = fname+"-max_component"
        if( params['cut'] ):
            fname = fname+"-edge_cut"
        if( params['graphml'] ):
            #fname = fname+".gml"
            ### NOTE: THIS FIX MUST BE APPLIED !!!
            fname = fname+".graphml"
            if( params['report'] ):
                print "Writing GraphML file:",fname
            nx.write_graphml(G=g, 
                             path=fname,
                             encoding='utf-8', 
                             prettyprint=True)
        else:
            fname = fname+".dot"
            if( params['report'] ):
                print "Writing 'dot' file:",fname
            write_dot_file(graph=g,fname=fname)
            
    # Always remember to close the DB when you're done
    db.close()
    return


if __name__ == '__main__':
    main(sys.argv)

