#!/usr/bin/python
# -*- coding: utf-8 -*-
# 
#   FILE: ClassifyTweet.py
#   DATE: March, 2016
#   Author: David W. McDonald
#
#   A simple machine learning object, based on resources from nltk, that can
#   learn to classify tweets, with some tweet specific items 
#
#   Copyright by Author. All rights reserved. Not for reuse without
#   express permissions.
#
#
import sys, string, re, csv, nltk
from datetime import datetime, timedelta
from hcde.utils.tweet_entities import tweet_entities
from hcde.utils.stop_words import remove_stops, STOPLIST
from hcde.ml.Classification import Classification


class ClassifyTweet(Classification):
    def __init__(self):
        Classification.__init__(self)
        
        # Configuration options
        self.count_capitals = True
        self.ignore_case = True
        self.remove_punctuation = True
        self.remove_prepositions = True
        self.remove_conjunctions = True
        self.remove_numbers = True
        self.count_emoticons = True
        self.happysad_emoticons = True
        
        # Local data for making some options work
        self.punctuation = ".,;:?!*&^%$#@()[]{}-+='\"\\|/`~"
        #self.punctuation = string.punctuation
        #self.punct_trans_table = string.maketrans("","")
        self.punct_trans_regex = re.compile('[%s]'%re.escape(self.punctuation))
        self.prepositions = ["of", "in", "to", "for", "with", "on", "at", "from", 
                             "by", "about", "as", "into", "like", "through", "after", 
                             "over", "between", "out", "against", "during", "without", 
                             "before", "under", "around", "among" ]
        self.conjunctions = ["and", "that", "but", "or", "as", "if", "when", "than", 
                             "because", "while", "where", "after", "so", "though", 
                             "since", "until", "whether", "before", "although", "nor", 
                             "like", "once", "unless", "now", "except"]
        # dictionary of emoticons where the dict keys are the emoticon characters
        # and the dict values are a textual description of the emoticon 
        self.emoticons = { "(.V.)":"Alien", "O:-)":"Angel", "X-(":"Angry", "~:0":"Baby", 
                           ":-D":"Big Grin", "(*v*)":"Bird", ":-#":"Braces", 
                           "</3":"Broken Heart (2)", "=^.^=":"Cat", "*<:o)":"Clown", 
                           "O.o":"Confused", "B-)":"Cool", ":_(":"Crying (3)", 
                           "\:D/":"Dancing", "*-*":"Dazed", ":o3":"Dog", "#-o":"Doh!", 
                           ":*)":"Drunk", "//_^":"Emo", ">:)":"Evil Grin", "<><":"Fish", 
                           ":-(":"Frown (2)", ":-(":"Frowning", "=P":"Frustrated (2)", 
                           "8-)":"Glasses", "$_$":"Greedy", ":->":"Grin", "=)":"Happy (3)", 
                           "#":"Hashtag", "<3":"Heart", "{}":"Hug", ":-|":"Indifferent", 
                           "X-p":"Joking", ":-*":"Kiss (3)", "(-}{-)":"Kissing", 
                           "XD":"Laughing", "=D":"Laughing Out Loud", ")-:":"Left-handed Sad Face", 
                           "(-:":"Left-handed Smiley Face", "<3":"Love", "=/":"Mad", 
                           ":-)(-:":"Married", "@":"Mention", "<:3)~":"Mouse", "~,~":"Napping", 
                           ":-B":"Nerd", "^_^":"Overjoyed", "<l:0":"Partying", ":-/":"Perplexed", 
                           "=8)":"Pig", "@~)~~~~":"Rose", ":(":"Sad (3)", ":S":"Sarcastic", 
                           ":-@":"Screaming", "=O":"Shocked (2)", ":)":"Smile (2)", 
                           ":-Q":"Smoking", ":>":"Smug", ":P":"Sticking Tongue Out", 
                           ":o":"Surprised", ":-J":"Tongue in Cheek", ":-&":"Tongue Tied", 
                           "=-O":"Uh-oh", ":-\\":"Undecided", ":/":"Unsure", ":-E":"Vampire", 
                           "=D":"Very Happy", ";-)":"Winking (2)", "|-O":"Yawn", "8-#":"Zombie"}
    
    
    def parse_features(self, text=""):
        feature_dict = dict()
        # the load method can return lists or strings this allows the
        # routine to work with either - just convert to what we need
        # at the start
        if( type(text) is list ):
            text = u" ".join(text)
        
        # if we need to count the number of capitals, better do that first
        if( self.count_capitals ):
            cap_count = sum(1 for c in text if c.isupper())
        else:
            cap_count = 0
        
        # if we are to ignore the case then make it all lower case
        if( self.ignore_case ):
            text = text.lower()
        
        # parse the tweet entities to save these as features
        ent = tweet_entities(tweet_text=text)
        
        # as long as the whole thing isn't just tabs, spaces, whitespace
        # then go on to process the text
        if( not ent['is_whitespace'] ):
            # keep a set of features that are extracted from the
            # tweet_entities() parser, these are really tweet specific
            feature_dict['is_retweet'] = ent['is_retweet']
            feature_dict['is_short'] = ent['is_short']
            if( ent['hashes'] ):
                feature_dict['hashtag_count'] = len(ent['hashes'])
            if( ent['mentions'] ):
                feature_dict['mention_count'] = len(ent['mentions'])
            if( ent['urls'] ):
                feature_dict['url_count'] = len(ent['urls'])
            if( self.count_capitals ):
                feature_dict['cap_count'] = cap_count
            
            # check to see if we should count or keep the emoticons
            # this has to be done first, because later we strip out
            # anything that looks like punctuation - after that all the
            # emoticons would disappear
            if( self.count_emoticons or self.happysad_emoticons ):
                # need to check for emoticons, that is not handled by the
                # tweet_entities() routine, so we need to do this now, before
                # we start throwing away punctuation
                #
                # tokenize the original string
                word_list = text.split()
                for tok in word_list:
                    # now check to see if we're adding the emoticon count as a
                    # feature, and if this token is one of the recognized emoticons
                    if( tok in self.emoticons ):
                        # maybe specific emoticons are features, like those we can
                        # identify as meaning something like happy or sad
                        if( self.happysad_emoticons ):
                            if( (self.emoticons[tok].find('Happy')>=0) or
                                (self.emoticons[tok].find('Sad')>=0) or
                                (self.emoticons[tok].find('Frown')>=0) or
                                (self.emoticons[tok].find('Smil')>=0) ):
                                feature_dict[tok] = (tok in self.complete_token_list)
                        # keep a count of the total number of emoticons
                        if( self.count_emoticons ):
                            if( 'emoticon_count' in feature_dict ):
                                feature_dict['emoticon_count'] += 1
                            else:
                                feature_dict['emoticon_count'] = 1
            
            # there are a couple options here, the first line uses
            # exactly the text passed in, the second line (currently active)
            # will use the cleaned up text of the tweet coming back from the
            # tweet_entities() routine.
            #t2 = text
            t2 = ent['fixed_tweet_text']
            
            # this removes all punctuation listed in the string self.punctuation
            # strips it all out replacing punctuation with nothing
            # the technique using string.translate() does not seem to work correctly
            #t2 = t2.translate(self.punct_trans_table,self.punctuation)
            #t2 = t2.translate(string.maketrans("",""), self.punctuation)
            #
            # this substitute technique using a regular expression will remove
            # all of the punctuation, almost like the translate function, but works
            if( self.remove_punctuation ):
                t2 = self.punct_trans_regex.sub('', t2)
            
            # tokenize the string
            word_list = t2.split()
            
            # the tweet_entities strip out these special items 
            word_list.extend(ent['hashes'])
            word_list.extend(ent['mentions'])
            
            # the string was converted to a list of tokens, run through each
            # token and figure out if it is a type of feature we want to keep
            for tok in word_list:
                # first make sure this token is not whitespace (a space or tab)
                if( tok ):
                    
                    # now check the configuration for numbers
                    if( self.remove_numbers ):
                        try:
                            # if this succeeds, then tok is a number, we want
                            # to remove numbers, they are probably not good features
                            float(tok)
                            continue
                        except ValueError:
                            # a conversion to a number didn't work
                            # that's actually good, we want to remove numbers
                            pass
                    
                    # now check the configuration for prepositions                    
                    if( self.remove_prepositions and (tok in self.prepositions) ):
                        continue
                    
                    # now check the configuration for prepositions                    
                    if( self.remove_conjunctions and (tok in self.conjunctions) ):
                        continue
                    
                    feature_dict[tok] = (tok in self.complete_token_list)
        return feature_dict
    


if __name__ == '__main__':
    print "The object ClassifyTweet() can not be manipulated from the command line."

